"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const glob = require("glob");
const deps_1 = require("./deps");
const util_1 = require("util");
const path_1 = require("path");
async function tsGen(cfg, _plugins, _deps) {
    const deps = _deps || deps_1.createDeps();
    const plugins = util_1.isArray(_plugins) ? _plugins : [_plugins];
    const { cwd, prettier } = cfg;
    const { fs, logger } = deps;
    for (const plugin of plugins) {
        logger.info("Running before run");
        processOutput(deps, prettier, await plugin.beforeRun());
        const filePaths = glob.sync(plugin.ctx.rawConfig.files, { ignore: "node_modules/**", absolute: true, cwd });
        const fileDescs = filePaths.map(path => ({
            path,
            contents: fs.readFileSync(path, "utf8"),
        }));
        for (const fd of fileDescs) {
            logger.info(`Processing ${fd.path} with ${plugin.name} plugin`);
            processOutput(deps, prettier, await plugin.transformFile(fd));
        }
        logger.info("Running after run");
        processOutput(deps, prettier, await plugin.afterRun());
    }
}
exports.tsGen = tsGen;
function processOutput({ fs, prettier, logger, mkdirp }, prettierCfg, output) {
    if (!output) {
        return;
    }
    const outputFds = util_1.isArray(output) ? output : [output];
    outputFds.forEach(fd => {
        // ensure directory first
        mkdirp(path_1.dirname(fd.path));
        logger.info("Writing file: ", fd.path);
        fs.writeFileSync(fd.path, prettier.format(fd.contents, Object.assign({}, (prettierCfg || {}), { parser: "typescript" })), "utf8");
    });
}
exports.processOutput = processOutput;
